var react = require('react');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var POPSTATE = "popstate";
var location = document.location;
var addEventListener = window.addEventListener.bind(window);
var removeEventListener = window.removeEventListener.bind(window);
var dispatchEvent = window.dispatchEvent.bind(window);
var history = window.history;
var pushState = function pushState(path, state) {
  if (state === void 0) {
    state = null;
  }

  return history.pushState(state, "", path);
};
var replaceState = function replaceState(path, state) {
  if (state === void 0) {
    state = null;
  }

  return history.replaceState(state, "", path);
};
function pipe(v) {
  var ret = v;

  for (var _len = arguments.length, funcs = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    funcs[_key - 1] = arguments[_key];
  }

  for (var _i = 0, _funcs = funcs; _i < _funcs.length; _i++) {
    var f = _funcs[_i];
    ret = f(ret);
  }

  return ret;
}
function arrEq(a1, a2) {
  if (a1.length !== a2.length) return false;
  return a1.every(function (v, i) {
    return v === a2[i];
  });
}
function makeEvent(eventName) {
  return typeof Event === "function" ? new Event(eventName) : pipe(document.createEvent(eventName), function (e) {
    e.initEvent(eventName, true, true);
    return e;
  });
}
function parsePath(raw) {
  switch (raw) {
    case "":
    case "/":
      return [];

    default:
      return pipe(raw, function (raw) {
        return raw.slice(1);
      }, function (raw) {
        return raw[raw.length - 1] === "/" ? raw.slice(0, -1) : raw;
      }, function (raw) {
        return raw.split("?")[0];
      }, function (raw) {
        return raw.split("/").filter(function (s) {
          return s.length !== 0;
        });
      });
  }
}
function parseHash(raw) {
  switch (raw) {
    case "":
    case "#":
      return "";

    default:
      return raw.slice(1);
  }
}
function parseSearch(raw) {
  switch (raw) {
    case "":
    case "?":
      return "";

    default:
      return raw.split("?")[1];
  }
}
function safeStringify(state) {
  var ret = null;

  try {
    ret = JSON.stringify(state);
  } catch (e) {
    console.error(e);
  }

  return ret;
}
function safeParse(state) {
  var ret = null;

  try {
    ret = JSON.parse(state);
  } catch (e) {
    console.error(e);
  }

  return ret;
}

function push(path, state, baseUrl) {
  if (state === void 0) {
    state = null;
  }

  if (baseUrl === void 0) {
    baseUrl = "";
  }

  pushState(baseUrl + path, safeStringify(state));
  dispatchEvent(makeEvent(POPSTATE));
}
function replace(path, state, baseUrl) {
  if (state === void 0) {
    state = null;
  }

  if (baseUrl === void 0) {
    baseUrl = "";
  }

  replaceState(baseUrl + path, safeStringify(state));
  dispatchEvent(makeEvent(POPSTATE));
}
function getInitUrl() {
  var state = typeof history.state === "string" ? history.state : safeStringify(history.state);
  return {
    path: path(),
    hash: hash(),
    search: search(),
    state: state
  };
}
function watchUrl(callback) {
  var watchID = function watchID() {
    return callback(getInitUrl());
  };

  addEventListener(POPSTATE, watchID);
  return watchID;
}
function unwatchUrl(watchID) {
  removeEventListener(POPSTATE, watchID);
}
function useUrl() {
  var _useState = react.useState(getInitUrl),
      url = _useState[0],
      setUrl = _useState[1];

  react.useEffect(function () {
    var watchID = watchUrl(function (url) {
      return setUrl(function () {
        return url;
      });
    });
    var newUrl = getInitUrl();

    if (!urlEqual(newUrl, url)) {
      setUrl(function () {
        return newUrl;
      });
    }

    return function () {
      return unwatchUrl(watchID);
    };
  });
  return _extends({}, url, {
    state: safeParse(url.state)
  });
}
var otherwise = Symbol("otherwise");
function matchPath(path, matchOption) {
  var _matchOption$otherwis;

  for (var _i = 0, _Object$keys = Object.keys(matchOption); _i < _Object$keys.length; _i++) {
    var key = _Object$keys[_i];
    var matcher = parsePath(key);
    var ret = match(path, matcher);

    switch (ret.t) {
      case "NotMatch":
        continue;

      case "Match":
        return matchOption[key].apply(matchOption, ret.vars);
    }
  }

  return ((_matchOption$otherwis = matchOption[otherwise]) != null ? _matchOption$otherwis : function () {})();
}

function match(realPath, matcher) {
  var len = realPath.length;
  if (len !== matcher.length) return {
    t: "NotMatch"
  };
  var matchedVar = [];

  for (var i = 0; i < len; i++) {
    var p = realPath[i];
    var mp = matcher[i];

    if (mp.startsWith(":")) {
      matchedVar.push(p);
    } else if (p !== mp) {
      return {
        t: "NotMatch"
      };
    }
  }

  return {
    t: "Match",
    vars: matchedVar
  };
}

function urlEqual(u1, u2) {
  return u1.hash === u2.hash && u1.search === u2.search && arrEq(u1.path, u2.path) && u1.state === u2.state;
}

function path() {
  return parsePath(location.pathname);
}

function hash() {
  return parseHash(location.hash);
}

function search() {
  return parseSearch(location.search);
}

exports.getInitUrl = getInitUrl;
exports.matchPath = matchPath;
exports.otherwise = otherwise;
exports.push = push;
exports.replace = replace;
exports.unwatchUrl = unwatchUrl;
exports.useUrl = useUrl;
exports.watchUrl = watchUrl;
//# sourceMappingURL=index.js.map
